@@ -124,6 +124,10 @@ const char* stats_title = " Statistics ";
 
 /* Footer labels. */
 const char* main_window_footer = "S=Start M=Method P=PRNG V=Verify R=Rounds B=Blanking Space=Select Ctrl-C=Quit";
+const char* main_window_footer_warning_lower_case_s =
+    "  WARNING: To start the wipe press capital S, you pressed lower case s  ";
+const char* main_window_footer_warning_no_drive_selected =
+    "  No drives selected, use spacebar to select a drive, then press S to start  ";
 const char* selection_footer = "J=Down K=Up Space=Select Backspace=Cancel Ctrl-C=Quit";
 const char* end_wipe_footer = "B=Blank screen Ctrl-C=Quit";
 const char* rounds_footer = "Left=Erase Esc=Cancel Ctrl-C=Quit";
@@ -213,12 +217,21 @@ void nwipe_gui_init( void )
         /* Set white on green for success messages. */
         init_pair( 5, COLOR_WHITE, COLOR_GREEN );
 
-        /* Set white on green for failure messages. */
+        /* Set white on red for failure messages. */
         init_pair( 6, COLOR_WHITE, COLOR_RED );
 
         /* Set black on black for when hiding the display. */
         init_pair( 7, COLOR_BLACK, COLOR_BLACK );
 
+        /* Set green on blue for reverse bold messages */
+        init_pair( 8, COLOR_GREEN, COLOR_WHITE );
+
+        /* Set green on blue for reverse bold error messages */
+        init_pair( 9, COLOR_RED, COLOR_WHITE );
+
+        /* Set black on yellow for warning messages */
+        init_pair( 10, COLOR_BLACK, COLOR_YELLOW );
+
         /* Set the background style. */
         wbkgdset( stdscr, COLOR_PAIR( 1 ) | ' ' );
     }
@@ -517,7 +530,7 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
     int focus = 0;
 
     /* A generic loop variable. */
-    int i;
+    int i = 0;
 
     /* User input buffer. */
     int keystroke;
@@ -528,6 +541,12 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
     /* Flag, Valid key hit = 1, anything else = 0 */
     int validkeyhit;
 
+    /* counts number of drives and partitions that have been selected */
+    int number_of_selected_contexts = 0;
+
+    /* Control A toggle status -1=indefined, 0=all drives delected, 1=all drives selected */
+    int select_all_toggle_status = -1;
+
     /* Get the terminal size */
     getmaxyx( stdscr, stdscr_lines, stdscr_cols );
 
@@ -545,6 +564,38 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
 
         /* Less two lines for the box and two lines for padding. */
         slots = wlines - 4;
+        if( slots < 0 )
+        {
+            slots = 0;
+        }
+
+        /* The code here adjusts the offset value, required when the terminal is resized vertically */
+        if( slots > count )
+        {
+            offset = 0;
+        }
+        else
+        {
+            if( focus >= count )
+            {
+                /* The focus is already at the last element. */
+                focus = count - 1;
+            }
+            if( focus < 0 )
+            {
+                /* The focus is already at the last element. */
+                focus = 0;
+            }
+        }
+
+        if( count >= slots && slots > 0 )
+        {
+            offset = focus + 1 - slots;
+            if( offset < 0 )
+            {
+                offset = 0;
+            }
+        }
 
         /* Clear the main window, necessary when switching selections such as method etc */
         werase( main_window );
@@ -595,42 +646,59 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
                 waddch( main_window, ' ' );
             }
 
-            switch( c[i + offset]->select )
+            /* In the event of the offset value some how becoming invalid, this if statement will prevent a segfault
+             * and the else part will log the out of bounds values for debugging */
+            if( i + offset >= 0 && i + offset < count )
             {
-                case NWIPE_SELECT_TRUE:
 
-                    wprintw( main_window, " [wipe] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
-                    break;
+                switch( c[i + offset]->select )
+                {
+                    case NWIPE_SELECT_TRUE:
 
-                case NWIPE_SELECT_FALSE:
-                    /* Print an element that is not selected. */
-                    wprintw( main_window, " [    ] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
-                    break;
+                        wprintw( main_window, " [wipe] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
+                        break;
 
-                case NWIPE_SELECT_TRUE_PARENT:
+                    case NWIPE_SELECT_FALSE:
+                        /* Print an element that is not selected. */
+                        wprintw( main_window, " [    ] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
+                        break;
 
-                    /* This element will be wiped when its parent is wiped. */
-                    wprintw( main_window, " [****] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
-                    break;
+                    case NWIPE_SELECT_TRUE_PARENT:
 
-                case NWIPE_SELECT_FALSE_CHILD:
+                        /* This element will be wiped when its parent is wiped. */
+                        wprintw( main_window, " [****] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
+                        break;
 
-                    /* We can't wipe this element because it has a child that is being wiped. */
-                    wprintw( main_window, " [----] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
-                    break;
+                    case NWIPE_SELECT_FALSE_CHILD:
 
-                case NWIPE_SELECT_DISABLED:
+                        /* We can't wipe this element because it has a child that is being wiped. */
+                        wprintw( main_window, " [----] %i. %s", ( i + offset + 1 ), c[i + offset]->device_label );
+                        break;
 
-                    /* We don't know how to wipe this device. (Iomega Zip drives.) */
-                    wprintw( main_window, " [????] %s", "Unrecognized Device" );
-                    break;
+                    case NWIPE_SELECT_DISABLED:
 
-                default:
+                        /* We don't know how to wipe this device. (Iomega Zip drives.) */
+                        wprintw( main_window, " [????] %s", "Unrecognized Device" );
+                        break;
 
-                    /* TODO: Handle the sanity error. */
-                    break;
+                    default:
 
-            } /* switch select */
+                        /* TODO: Handle the sanity error. */
+                        break;
+
+                } /* switch select */
+            }
+            else
+            {
+                nwipe_log( NWIPE_LOG_DEBUG,
+                           "GUI.c,nwipe_gui_select(), scroll, array index out of bounds, i=%u, count=%u, slots=%u, "
+                           "focus=%u, offset=%u",
+                           i,
+                           count,
+                           slots,
+                           focus,
+                           offset );
+            }
 
         } /* for */
 
@@ -666,20 +734,27 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
          * which wastes CPU cycles.
          */
 
-        validkeyhit = 0;
-
         do
         {
+            validkeyhit = 0;
             timeout( 250 );  // block getch() for 250ms.
             keystroke = getch();  // Get user input.
             timeout( -1 );  // Switch back to blocking mode.
 
+            /* We don't necessarily use these but for future reference, here are some cntrl + key values
+             * ^W - 23, ^E - 5, ^R - 18, ^T - 20, ^Y - 25, ^U - 21, ^I - 9, ^O - 15, ^P - 16, ^A - 1, ^D - 4
+             * ^F - 6, ^G - 7, ^H - 8, ^K - 11, ^L - 12, ^X - 24, ^V - 22, ^B - 2, ^N - 14
+             * Use nwipe_log( NWIPE_LOG_DEBUG, "Key Name: %s - %u", keyname(keystroke),keystroke) to
+             * figure out what code is returned by what ever key combination */
+
             switch( keystroke )
             {
                 case KEY_DOWN:
                 case 'j':
                 case 'J':
 
+                    validkeyhit = 1;
+
                     /* Increment the focus. */
                     focus += 1;
 
@@ -696,7 +771,6 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
                         offset += 1;
                         break;
                     }
-                    validkeyhit = 1;
 
                     break;
 
@@ -704,6 +778,8 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
                 case 'k':
                 case 'K':
 
+                    validkeyhit = 1;
+
                     /* Decrement the focus. */
                     focus -= 1;
 
@@ -720,7 +796,6 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
                         offset -= 1;
                         break;
                     }
-                    validkeyhit = 1;
 
                     break;
 
@@ -728,6 +803,8 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
                 case 10:
                 case ' ':
 
+                    validkeyhit = 1;
+
                     /* TODO: This block should be made into a function. */
 
                     if( c[focus]->select == NWIPE_SELECT_TRUE )
@@ -794,8 +871,6 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
 
                         } /* else super-enable */
 
-                        validkeyhit = 1;
-
                         break;
 
                     } /* if NWIPE_SELECT_TRUE */
@@ -842,60 +917,164 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
 
                         } /* else super-deselect */
 
-                        validkeyhit = 1;
-
                         break;
 
                     } /* if NWIPE_SELECT_FALSE */
 
                     /* TODO: Explain to the user why they can't change this. */
-                    validkeyhit = 1;
                     break;
 
                 case 'm':
                 case 'M':
 
+                    validkeyhit = 1;
+
                     /*  Run the method dialog. */
                     nwipe_gui_method();
-                    validkeyhit = 1;
                     break;
 
                 case 'p':
                 case 'P':
 
+                    validkeyhit = 1;
+
                     /* Run the PRNG dialog. */
                     nwipe_gui_prng();
-                    validkeyhit = 1;
+
                     break;
 
                 case 'r':
                 case 'R':
 
+                    validkeyhit = 1;
+
                     /* Run the rounds dialog. */
                     nwipe_gui_rounds();
-                    validkeyhit = 1;
+
                     break;
 
                 case 'v':
                 case 'V':
 
+                    validkeyhit = 1;
+
                     /* Run the option dialog. */
                     nwipe_gui_verify();
-                    validkeyhit = 1;
                     break;
 
                 case 'b':
                 case 'B':
 
+                    validkeyhit = 1;
+
                     /* Run the noblank dialog. */
                     nwipe_gui_noblank();
-                    validkeyhit = 1;
                     break;
 
                 case 'S':
 
-                    /* User want to start the wipe */
+                    /* User wants to start the wipe */
                     validkeyhit = 1;
+
+                    /* Have any drives have been selected ? */
+                    number_of_selected_contexts = 0;
+                    for( i = 0; i < count; i++ )
+                    {
+                        if( c[i]->select == NWIPE_SELECT_TRUE )
+                        {
+                            number_of_selected_contexts += 1;
+                        }
+                    }
+
+                    /* if no drives have been selected, print a warning on the footer */
+                    if( number_of_selected_contexts == 0 )
+                    {
+                        wattron( footer_window, COLOR_PAIR( 10 ) );
+                        nwipe_gui_amend_footer_window( main_window_footer_warning_no_drive_selected );
+                        doupdate();
+                        sleep( 3 );
+                        wattroff( footer_window, COLOR_PAIR( 10 ) );
+
+                        /* After the delay return footer text back to key help */
+                        nwipe_gui_amend_footer_window( main_window_footer );
+                        doupdate();
+
+                        /* Remove any repeated S key strokes, without this the gui would hang
+                         * for a period of time, i.e sleep above x number of repeated 's' keystrokes
+                         * which could run into minutes */
+                        do
+                        {
+                            timeout( 250 );  // block getch() for 250ms.
+                            keystroke = getch();  // Get user input.
+                            timeout( -1 );  // Switch back to blocking mode.
+                        } while( keystroke == 'S' );
+
+                        /* Remove the S from keystroke, which allows us to stay within the selection menu loop */
+                        keystroke = 0;
+                    }
+
+                    break;
+
+                case 's':
+
+                    /* user has mistakenly hit the lower case 's' instead of capital 'S' */
+                    validkeyhit = 1;
+
+                    /* Warn the user about their mistake */
+                    wattron( footer_window, COLOR_PAIR( 10 ) );
+                    nwipe_gui_amend_footer_window( main_window_footer_warning_lower_case_s );
+                    doupdate();
+                    sleep( 3 );
+                    wattroff( footer_window, COLOR_PAIR( 10 ) );
+
+                    /* After the delay return footer text back to key help */
+                    nwipe_gui_amend_footer_window( main_window_footer );
+                    doupdate();
+
+                    /* Remove any repeated s key strokes, without this the gui would hang
+                     * for a period of time, i.e sleep above x number of repeated 's' keystrokes
+                     * which could run into minutes */
+                    do
+                    {
+                        timeout( 250 );  // block getch() for 250ms.
+                        keystroke = getch();  // Get user input.
+                        timeout( -1 );  // Switch back to blocking mode.
+                    } while( keystroke == 's' );
+
+                    break;
+
+                case 1:
+
+                    /* Ctrl A - Toggle select/deselect all drives */
+                    validkeyhit = 1;
+
+                    if( select_all_toggle_status == -1 || select_all_toggle_status == 0 )
+                    {
+                        for( i = 0; i < count; i++ )
+                        {
+                            c[i]->select = NWIPE_SELECT_TRUE;
+                        }
+                        select_all_toggle_status = 1;
+                    }
+                    else
+                    {
+                        if( select_all_toggle_status == 1 )
+                        {
+                            for( i = 0; i < count; i++ )
+                            {
+                                c[i]->select = NWIPE_SELECT_FALSE;
+                            }
+                            select_all_toggle_status = 0;
+                        }
+                        else
+                        {
+                            nwipe_log(
+                                NWIPE_LOG_ERROR,
+                                "gui.c:nwipe_gui_select(), Invalid value in variable select_all_toggle_status = %d",
+                                select_all_toggle_status );
+                        }
+                    }
+
                     break;
 
             } /* keystroke switch */
@@ -910,6 +1089,14 @@ void nwipe_gui_select( int count, nwipe_context_t** c )
 
     } while( keystroke != 'S' && terminate_signal != 1 );
 
+    if( keystroke == 'S' )
+    {
+        /* If user has pressed S to start wipe change status line */
+        werase( footer_window );
+        nwipe_gui_title( footer_window, end_wipe_footer );
+        wnoutrefresh( footer_window );
+    }
+
 } /* nwipe_gui_select */
 
 void nwipe_gui_options( void )
@@ -2079,15 +2266,10 @@ void* nwipe_gui_status( void* ptr )
     nwipe_misc_thread_data = nwipe_thread_data_ptr->nwipe_misc_thread_data;
     count = nwipe_misc_thread_data->nwipe_selected;
 
-    /* Throughput print formats. */
-    char* nwipe_tera = "%llu TB/s";
-    char* nwipe_giga = "%llu GB/s";
-    char* nwipe_mega = "%llu MB/s";
-    char* nwipe_kilo = "%llu KB/s";
-    char* nwipe_unit = "%llu B/s";
+    char nomenclature_result_str[NOMENCLATURE_RESULT_STR_SIZE]; /* temporary usage */
 
-    /* The throughput format pointer. */
-    char* nwipe_format;
+    /* Spinner character */
+    char spinner_string[2];
 
     /* We count time from when this function is first called. */
     static time_t nwipe_time_start = 0;
@@ -2143,7 +2325,6 @@ void* nwipe_gui_status( void* ptr )
 
     /* Throughput variables */
     u64 nwipe_throughput;
-    u64 nwipe_throughput_stopped;
 
     /* The number of active wipe processes. */
     /* Set to 1 initially to start loop.    */
@@ -2317,7 +2498,7 @@ void* nwipe_gui_status( void* ptr )
                 case 0x0a:
 
                     /* Check whether we have finished all wipes, if yes exit while loop if user pressed spacebar or
-                     * return */
+                     * return. */
                     if( !nwipe_active || terminate_signal == 1 )
                     {
                         loop_control = 0;
@@ -2333,11 +2514,21 @@ void* nwipe_gui_status( void* ptr )
 
         } /* keystroke */
 
+        /* if wipe has completed and user has specified auto poweroff or nowait then we can skip waiting for the user to
+         * press return */
+        if( !nwipe_active )
+        {
+            if( nwipe_options.autopoweroff || nwipe_options.nowait )
+            {
+                loop_control = 0;
+            }
+        }
+
         /* Update screen if not blanked. */
         if( nwipe_gui_blank == 0 )
         {
 
-            if( nwipe_active && terminate_signal != 1 )
+            if( terminate_signal != 1 )
             {
                 nwipe_active = compute_stats( ptr );  // Returns number of active wipe threads
             }
@@ -2355,7 +2546,7 @@ void* nwipe_gui_status( void* ptr )
                     mvwprintw( main_window,
                                yy++,
                                4,
-                               "[%05.2f%%, round %i of %i, pass %i of %i] ",
+                               "[%5.2f%%, round %i of %i, pass %i of %i] ",
                                c[i]->round_percent,
                                c[i]->round_working,
                                c[i]->round_count,
@@ -2371,11 +2562,15 @@ void* nwipe_gui_status( void* ptr )
                     }
                     else if( c[i]->signal )
                     {
+                        wattron( main_window, COLOR_PAIR( 9 ) );
                         mvwprintw( main_window, yy++, 4, "(>>> FAILURE! <<<, signal %i) ", c[i]->signal );
+                        wattroff( main_window, COLOR_PAIR( 9 ) );
                     }
                     else
                     {
+                        wattron( main_window, COLOR_PAIR( 9 ) );
                         mvwprintw( main_window, yy++, 4, "(>>>FAILURE!<<<, code %i) ", c[i]->result );
+                        wattroff( main_window, COLOR_PAIR( 9 ) );
                     }
 
                 } /* child returned */
@@ -2388,57 +2583,71 @@ void* nwipe_gui_status( void* ptr )
                 {
                     wprintw( main_window, "[pass errors: %llu] ", c[i]->pass_errors );
                 }
-
-                switch( c[i]->pass_type )
+                if( c[i]->wipe_status == 1 )
                 {
-                    case NWIPE_PASS_FINAL_BLANK:
-                        wprintw( main_window, "[blanking] " );
-                        break;
+                    switch( c[i]->pass_type )
+                    {
+                        /* Each text field in square brackets should be the same number of characters
+                         * to retain output in columns */
+                        case NWIPE_PASS_FINAL_BLANK:
+                            if( !c[i]->sync_status )
+                            {
+                                wprintw( main_window, "[ blanking] " );
+                            }
+                            break;
 
-                    case NWIPE_PASS_FINAL_OPS2:
-                        wprintw( main_window, "[OPS-II final] " );
-                        break;
+                        case NWIPE_PASS_FINAL_OPS2:
+                            if( !c[i]->sync_status )
+                            {
+                                wprintw( main_window, "[OPS2final] " );
+                            }
+                            break;
 
-                    case NWIPE_PASS_WRITE:
-                        wprintw( main_window, "[writing] " );
-                        break;
+                        case NWIPE_PASS_WRITE:
+                            if( !c[i]->sync_status )
+                            {
+                                wprintw( main_window, "[ writing ] " );
+                            }
+                            break;
 
-                    case NWIPE_PASS_VERIFY:
-                        wprintw( main_window, "[verifying] " );
-                        break;
+                        case NWIPE_PASS_VERIFY:
+                            if( !c[i]->sync_status )
+                            {
+                                wprintw( main_window, "[verifying] " );
+                            }
+                            break;
 
-                    case NWIPE_PASS_NONE:
-                        break;
+                        case NWIPE_PASS_NONE:
+                            break;
+                    }
+
+                    if( c[i]->sync_status )
+                    {
+                        wprintw( main_window, "[ syncing ] " );
+                    }
                 }
 
-                if( c[i]->sync_status )
-                {
-                    wprintw( main_window, "[syncing] " );
-                }
+                /* Determine throughput nomenclature for this drive and output drives throughput to GUI */
+                Determine_C_B_nomenclature( c[i]->throughput, nomenclature_result_str, NOMENCLATURE_RESULT_STR_SIZE );
 
-                if( c[i]->throughput >= INT64_C( 1000000000000 ) )
-                {
-                    wprintw( main_window, "[%llu TB/s] ", c[i]->throughput / INT64_C( 1000000000000 ) );
-                }
-                else if( c[i]->throughput >= INT64_C( 1000000000 ) )
-                {
-                    wprintw( main_window, "[%llu GB/s] ", c[i]->throughput / INT64_C( 1000000000 ) );
-                }
-                else if( c[i]->throughput >= INT64_C( 1000000 ) )
-                {
-                    wprintw( main_window, "[%llu MB/s] ", c[i]->throughput / INT64_C( 1000000 ) );
-                }
-                else if( c[i]->throughput >= INT64_C( 1000 ) )
-                {
-                    wprintw( main_window, "[%llu KB/s] ", c[i]->throughput / INT64_C( 1000 ) );
-                }
-                else
-                {
-                    wprintw( main_window, "[%llu B/s] ", c[i]->throughput / INT64_C( 1 ) );
-                }
+                wprintw( main_window, "[%s/s] ", nomenclature_result_str );
 
                 /* Insert whitespace. */
                 yy += 1;
+
+                /* Increment the next spinner character for this context if the thread is active */
+                if( c[i]->wipe_status == 1 )
+                {
+                    spinner( c, i );
+                    spinner_string[0] = c[i]->spinner_character[0];
+                }
+                else
+                {
+                    /* if the wipe thread is no longer active, replace the spinner with a space */
+                    spinner_string[0] = ' ';
+                }
+                spinner_string[1] = 0;
+                wprintw( main_window, " %s ", spinner_string );
             }
 
             if( offset > 0 )
@@ -2463,48 +2672,18 @@ void* nwipe_gui_status( void* ptr )
             if( nwipe_active && terminate_signal != 1 )
             {
                 nwipe_gui_load();
-                nwipe_throughput = nwipe_misc_thread_data->throughput;
-                nwipe_throughput_stopped = nwipe_throughput;
-            }
-            else
-            {
-                nwipe_throughput = nwipe_throughput_stopped;
             }
 
-            if( nwipe_throughput >= INT64_C( 1000000000000 ) )
-            {
-                nwipe_throughput /= INT64_C( 1000000000000 );
-                nwipe_format = nwipe_tera;
-            }
-            else if( nwipe_throughput >= INT64_C( 1000000000 ) )
-            {
-                nwipe_throughput /= INT64_C( 1000000000 );
-                nwipe_format = nwipe_giga;
-            }
-            else if( nwipe_throughput >= INT64_C( 1000000 ) )
-            {
-                nwipe_throughput /= INT64_C( 1000000 );
-                nwipe_format = nwipe_mega;
-            }
-            else if( nwipe_throughput >= INT64_C( 1000 ) )
-            {
-                nwipe_throughput /= INT64_C( 1000 );
-                nwipe_format = nwipe_kilo;
-            }
-            else
-            {
-                nwipe_throughput /= INT64_C( 1 );
-                nwipe_format = nwipe_unit;
-            }
+            nwipe_throughput = nwipe_misc_thread_data->throughput;
+
+            /* Determine the nomenclature for the combined throughput */
+            Determine_C_B_nomenclature( nwipe_throughput, nomenclature_result_str, NOMENCLATURE_RESULT_STR_SIZE );
 
             /* Print the combined throughput. */
             mvwprintw( stats_window, NWIPE_GUI_STATS_THROUGHPUT_Y, NWIPE_GUI_STATS_THROUGHPUT_X, "Throughput:" );
 
-            if( nwipe_throughput > 0 )
-            {
-                mvwprintw(
-                    stats_window, NWIPE_GUI_STATS_THROUGHPUT_Y, NWIPE_GUI_STATS_TAB, nwipe_format, nwipe_throughput );
-            }
+            mvwprintw(
+                stats_window, NWIPE_GUI_STATS_THROUGHPUT_Y, NWIPE_GUI_STATS_TAB, "%s/s", nomenclature_result_str );
 
             /* Change the current time into a delta. */
             nwipe_time_now -= nwipe_time_start;
@@ -2551,7 +2730,7 @@ void* nwipe_gui_status( void* ptr )
             /* Print the error count. */
             mvwprintw( stats_window, NWIPE_GUI_STATS_ERRORS_Y, NWIPE_GUI_STATS_ERRORS_X, "Errors:" );
             mvwprintw(
-                stats_window, NWIPE_GUI_STATS_ERRORS_Y, NWIPE_GUI_STATS_TAB, "%llu", nwipe_misc_thread_data->errors );
+                stats_window, NWIPE_GUI_STATS_ERRORS_Y, NWIPE_GUI_STATS_TAB, "  %llu", nwipe_misc_thread_data->errors );
 
             /* Add a border. */
             box( stats_window, 0, 0 );
@@ -2616,37 +2795,43 @@ int compute_stats( void* ptr )
         {
             /* Increment the child counter. */
             nwipe_active += 1;
-        }
 
-        /* Even if the wipe has finished ALWAYS run the stats one last time so the final SUCCESS percentage value is
-         * correct. Maintain a rolling average of throughput. */
-        nwipe_update_speedring( &c[i]->speedring, c[i]->round_done, nwipe_time_now );
+            /* Even if the wipe has finished ALWAYS run the stats one last time so the final SUCCESS percentage value is
+             * correct. Maintain a rolling average of throughput. */
+            nwipe_update_speedring( &c[i]->speedring, c[i]->round_done, nwipe_time_now );
 
-        if( c[i]->speedring.timestotal > 0 && c[i]->wipe_status == 1 )
-        {
-            /* Update the current average throughput in bytes-per-second. */
-            c[i]->throughput = c[i]->speedring.bytestotal / c[i]->speedring.timestotal;
-
-            /* Update the estimated remaining runtime. */
-            /* Check that throughput is not zero (sometimes caused during a sync) */
-            if( c[i]->throughput == 0 )
+            if( c[i]->speedring.timestotal > 0 && c[i]->wipe_status == 1 )
             {
-                c[i]->throughput = 1;
+                /* Update the current average throughput in bytes-per-second. */
+                c[i]->throughput = c[i]->speedring.bytestotal / c[i]->speedring.timestotal;
+
+                /* Update the estimated remaining runtime. */
+                /* Check that throughput is not zero (sometimes caused during a sync) */
+                if( c[i]->throughput == 0 )
+                {
+                    c[i]->throughput = 1;
+                }
+
+                c[i]->eta = ( c[i]->round_size - c[i]->round_done ) / c[i]->throughput;
+
+                if( c[i]->eta > nwipe_misc_thread_data->maxeta )
+                {
+                    nwipe_misc_thread_data->maxeta = c[i]->eta;
+                }
             }
 
-            c[i]->eta = ( c[i]->round_size - c[i]->round_done ) / c[i]->throughput;
-
-            if( c[i]->eta > nwipe_misc_thread_data->maxeta )
-            {
-                nwipe_misc_thread_data->maxeta = c[i]->eta;
-            }
+            /* Calculate the average throughput */
+            c[i]->throughput = (double) c[i]->round_done / (double) difftime( nwipe_time_now, c[i]->start_time );
         }
 
         /* Update the percentage value. */
         c[i]->round_percent = (double) c[i]->round_done / (double) c[i]->round_size * 100;
 
-        /* Accumulate combined throughput. */
-        nwipe_misc_thread_data->throughput += c[i]->throughput;
+        if( c[i]->wipe_status == 1 )
+        {
+            /* Accumulate combined throughput. */
+            nwipe_misc_thread_data->throughput += c[i]->throughput;
+        }
 
         /* Accumulate the error count. */
         nwipe_misc_thread_data->errors += c[i]->pass_errors;
@@ -2694,3 +2879,30 @@ void nwipe_update_speedring( nwipe_speedring_t* speedring, u64 speedring_bytes,
         speedring->position = 0;
     }
 }
+
+int spinner( nwipe_context_t** ptr, int device_idx )
+{
+    nwipe_context_t** c;
+
+    c = ptr;
+
+    /* The spinner characters |/-\|/-\ */
+    char sc[9] = "|/-\\|/-\\/";
+
+    /* check sanity of index */
+    if( c[device_idx]->spinner_idx < 0 || c[device_idx]->spinner_idx > 7 )
+    {
+        return 1;
+    }
+
+    c[device_idx]->spinner_character[0] = sc[c[device_idx]->spinner_idx];
+
+    c[device_idx]->spinner_idx++;
+
+    if( c[device_idx]->spinner_idx > 7 )
+    {
+        c[device_idx]->spinner_idx = 0;
+    }
+
+    return 0;
+}
