@@ -123,6 +123,9 @@ void* nwipe_zero( void* ptr )
     nwipe_context_t* c;
     c = (nwipe_context_t*) ptr;
 
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
@@ -138,6 +141,9 @@ void* nwipe_zero( void* ptr )
     /* Finished. Set the wipe_status flag so that the GUI knows */
     c->wipe_status = 0;
 
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_zero */
 
@@ -150,6 +156,9 @@ void* nwipe_verify( void* ptr )
     nwipe_context_t* c;
     c = (nwipe_context_t*) ptr;
 
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
@@ -162,6 +171,9 @@ void* nwipe_verify( void* ptr )
     /* Finished. Set the wipe_status flag so that the GUI knows */
     c->wipe_status = 0;
 
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_verify */
 
@@ -175,6 +187,9 @@ void* nwipe_dod522022m( void* ptr )
     nwipe_context_t* c;
     c = (nwipe_context_t*) ptr;
 
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
@@ -230,6 +245,9 @@ void* nwipe_dod522022m( void* ptr )
     /* Finished. Set the wipe_status flag so that the GUI knows */
     c->wipe_status = 0;
 
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_dod522022m */
 
@@ -244,6 +262,9 @@ void* nwipe_dodshort( void* ptr )
     nwipe_context_t* c;
     c = (nwipe_context_t*) ptr;
 
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
@@ -292,6 +313,9 @@ void* nwipe_dodshort( void* ptr )
     /* Finished. Set the wipe_status flag so that the GUI knows */
     c->wipe_status = 0;
 
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_dodshort */
 
@@ -305,6 +329,9 @@ void* nwipe_gutmann( void* ptr )
     nwipe_context_t* c;
     c = (nwipe_context_t*) ptr;
 
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
@@ -426,6 +453,9 @@ void* nwipe_gutmann( void* ptr )
     /* Finished. Set the wipe_status flag so that the GUI knows */
     c->wipe_status = 0;
 
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_gutmann */
 
@@ -443,6 +473,9 @@ void* nwipe_ops2( void* ptr )
     nwipe_context_t* c;
     c = (nwipe_context_t*) ptr;
 
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
@@ -588,12 +621,19 @@ void* nwipe_ops2( void* ptr )
     /* Finished. Set the wipe_status flag so that the GUI knows */
     c->wipe_status = 0;
 
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_ops2 */
 
 void* nwipe_is5enh( void* ptr )
 {
     nwipe_context_t* c = (nwipe_context_t*) ptr;
+
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     c->wipe_status = 1;
 
     char is5enh[3] = {'\x00', '\xFF', '\x00'};
@@ -604,6 +644,10 @@ void* nwipe_is5enh( void* ptr )
     c->result = nwipe_runmethod( c, patterns );
 
     c->wipe_status = 0;
+
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_is5enh */
 
@@ -617,6 +661,9 @@ void* nwipe_random( void* ptr )
     nwipe_context_t* c;
     c = (nwipe_context_t*) ptr;
 
+    /* get current time at the start of the wipe  */
+    time( &c->start_time );
+
     /* set wipe in progress flag for GUI */
     c->wipe_status = 1;
 
@@ -629,6 +676,9 @@ void* nwipe_random( void* ptr )
     /* Finished. Set the wipe_status flag so that the GUI knows */
     c->wipe_status = 0;
 
+    /* get current time at the end of the wipe  */
+    time( &c->end_time );
+
     return NULL;
 } /* nwipe_random */
 
@@ -703,21 +753,16 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
     /* Initialize the working round counter. */
     c->round_working = 0;
 
-    nwipe_log( NWIPE_LOG_NOTICE,
-               "Invoking method '%s' on device '%s'.",
-               nwipe_method_label( nwipe_options.method ),
-               c->device_name );
+    nwipe_log(
+        NWIPE_LOG_NOTICE, "Invoking method '%s' on %s", nwipe_method_label( nwipe_options.method ), c->device_name );
 
     while( c->round_working < c->round_count )
     {
         /* Increment the round counter. */
         c->round_working += 1;
 
-        nwipe_log( NWIPE_LOG_NOTICE,
-                   "Starting round %i of %i on device '%s'.",
-                   c->round_working,
-                   c->round_count,
-                   c->device_name );
+        nwipe_log(
+            NWIPE_LOG_NOTICE, "Starting round %i of %i on %s", c->round_working, c->round_count, c->device_name );
 
         /* Initialize the working pass counter. */
         c->pass_working = 0;
@@ -738,7 +783,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
             }
 
             nwipe_log( NWIPE_LOG_NOTICE,
-                       "Starting pass %i of %i, round %i of %i, on device '%s'.",
+                       "Starting pass %i/%i, round %i/%i, on %s",
                        c->pass_working,
                        c->pass_count,
                        c->round_working,
@@ -761,7 +806,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
                 c->pass_type = NWIPE_PASS_NONE;
 
                 /* Log number of bytes written to disk */
-                nwipe_log( NWIPE_LOG_NOTICE, "%llu bytes written to device '%s'.", c->pass_done, c->device_name );
+                nwipe_log( NWIPE_LOG_NOTICE, "%llu bytes written to %s", c->pass_done, c->device_name );
 
                 /* Check for a fatal error. */
                 if( r < 0 )
@@ -773,7 +818,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
                 {
 
                     nwipe_log( NWIPE_LOG_NOTICE,
-                               "Verifying pass %i of %i, round %i of %i, on device '%s'.",
+                               "Verifying pass %i of %i, round %i of %i, on %s",
                                c->pass_working,
                                c->pass_count,
                                c->round_working,
@@ -792,7 +837,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
                     }
 
                     nwipe_log( NWIPE_LOG_NOTICE,
-                               "Verified pass %i of %i, round %i of %i, on device '%s'.",
+                               "Verified pass %i of %i, round %i of %i, on '%s'.",
                                c->pass_working,
                                c->pass_count,
                                c->round_working,
@@ -831,7 +876,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
                 c->pass_type = NWIPE_PASS_NONE;
 
                 /* Log number of bytes written to disk */
-                nwipe_log( NWIPE_LOG_NOTICE, "%llu bytes written to device '%s'.", c->pass_done, c->device_name );
+                nwipe_log( NWIPE_LOG_NOTICE, "%llu bytes written to %s", c->pass_done, c->device_name );
 
                 /* Check for a fatal error. */
                 if( r < 0 )
@@ -845,7 +890,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
                 if( nwipe_options.verify == NWIPE_VERIFY_ALL || lastpass == 1 || nwipe_options.method == &nwipe_is5enh )
                 {
                     nwipe_log( NWIPE_LOG_NOTICE,
-                               "Verifying pass %i of %i, round %i of %i, on device '%s'.",
+                               "Verifying pass %i of %i, round %i of %i, on %s",
                                c->pass_working,
                                c->pass_count,
                                c->round_working,
@@ -864,7 +909,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
                     }
 
                     nwipe_log( NWIPE_LOG_NOTICE,
-                               "Verified pass %i of %i, round %i of %i, on device '%s'.",
+                               "Verified pass %i of %i, round %i of %i, on '%s'.",
                                c->pass_working,
                                c->pass_count,
                                c->round_working,
@@ -875,7 +920,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
             } /* random pass */
 
             nwipe_log( NWIPE_LOG_NOTICE,
-                       "Finished pass %i of %i, round %i of %i, on device '%s'.",
+                       "Finished pass %i/%i, round %i/%i, on %s",
                        c->pass_working,
                        c->pass_count,
                        c->round_working,
@@ -884,11 +929,19 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
 
         } /* for passes */
 
-        nwipe_log( NWIPE_LOG_NOTICE,
-                   "Finished round %i of %i on device '%s'.",
-                   c->round_working,
-                   c->round_count,
-                   c->device_name );
+        if( c->round_working < c->round_count )
+        {
+            nwipe_log(
+                NWIPE_LOG_NOTICE, "Finished round %i of %i on %s", c->round_working, c->round_count, c->device_name );
+        }
+        else
+        {
+            nwipe_log( NWIPE_LOG_NOTICE,
+                       "Finished final round %i of %i on %s",
+                       c->round_working,
+                       c->round_count,
+                       c->device_name );
+        }
 
     } /* while rounds */
 
@@ -931,7 +984,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
 
         if( nwipe_options.verify == NWIPE_VERIFY_LAST || nwipe_options.verify == NWIPE_VERIFY_ALL )
         {
-            nwipe_log( NWIPE_LOG_NOTICE, "Verifying the final random pattern on '%s' is empty.", c->device_name );
+            nwipe_log( NWIPE_LOG_NOTICE, "Verifying the final random pattern on %s is empty.", c->device_name );
 
             /* Verify the final zero pass. */
             r = nwipe_random_verify( c );
@@ -951,7 +1004,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
 
     else if( nwipe_options.method == &nwipe_verify )
     {
-        nwipe_log( NWIPE_LOG_NOTICE, "Verifying that '%s' is empty.", c->device_name );
+        nwipe_log( NWIPE_LOG_NOTICE, "Verifying that %s is empty", c->device_name );
 
         /* Verify the final zero pass. */
         c->pass_type = NWIPE_PASS_VERIFY;
@@ -963,8 +1016,14 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
         {
             return r;
         }
-
-        nwipe_log( NWIPE_LOG_NOTICE, "Verified that '%s' is empty.", c->device_name );
+        if( c->verify_errors == 0 )
+        {
+            nwipe_log( NWIPE_LOG_NOTICE, "[SUCCESS] Verified that %s is empty.", c->device_name );
+        }
+        else
+        {
+            nwipe_log( NWIPE_LOG_ERROR, "[FAILURE] %s IS NOT empty.", c->device_name );
+        }
 
     } /* verify */
 
@@ -973,7 +1032,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
         /* Tell the user that we are on the final pass. */
         c->pass_type = NWIPE_PASS_FINAL_BLANK;
 
-        nwipe_log( NWIPE_LOG_NOTICE, "Blanking device '%s'.", c->device_name );
+        nwipe_log( NWIPE_LOG_NOTICE, "Blanking device %s", c->device_name );
 
         /* The final zero pass. */
         r = nwipe_static_pass( c, &pattern_zero );
@@ -986,7 +1045,7 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
 
         if( nwipe_options.verify == NWIPE_VERIFY_LAST || nwipe_options.verify == NWIPE_VERIFY_ALL )
         {
-            nwipe_log( NWIPE_LOG_NOTICE, "Verifying that '%s' is empty.", c->device_name );
+            nwipe_log( NWIPE_LOG_NOTICE, "Verifying that %s is empty.", c->device_name );
 
             /* Verify the final zero pass. */
             r = nwipe_static_verify( c, &pattern_zero );
@@ -997,10 +1056,17 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
                 return r;
             }
 
-            nwipe_log( NWIPE_LOG_NOTICE, "Verified that '%s' is empty.", c->device_name );
+            if( c->verify_errors == 0 )
+            {
+                nwipe_log( NWIPE_LOG_NOTICE, "[SUCCESS] Verified that %s is empty.", c->device_name );
+            }
+            else
+            {
+                nwipe_log( NWIPE_LOG_NOTICE, "[FAILURE] %s Verification errors, not empty", c->device_name );
+            }
         }
 
-        nwipe_log( NWIPE_LOG_NOTICE, "Blanked device '%s'.", c->device_name );
+        nwipe_log( NWIPE_LOG_NOTICE, "[SUCCESS] Blanked device %s", c->device_name );
 
     } /* final blank */
 
@@ -1014,13 +1080,13 @@ int nwipe_runmethod( nwipe_context_t* c, nwipe_pattern_t* patterns )
     if( c->verify_errors > 0 )
     {
         /* We finished, but with non-fatal verification errors. */
-        nwipe_log( NWIPE_LOG_ERROR, "%llu verification errors on device '%s'.", c->verify_errors, c->device_name );
+        nwipe_log( NWIPE_LOG_ERROR, "%llu verification errors on '%s'.", c->verify_errors, c->device_name );
     }
 
     if( c->pass_errors > 0 )
     {
         /* We finished, but with non-fatal wipe errors. */
-        nwipe_log( NWIPE_LOG_ERROR, "%llu wipe errors on device '%s'.", c->pass_errors, c->device_name );
+        nwipe_log( NWIPE_LOG_ERROR, "%llu wipe errors on '%s'.", c->pass_errors, c->device_name );
     }
 
     /* FIXME: The 'round_errors' context member is not being used. */
