@@ -19,6 +19,10 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
+#ifndef _DEFAULT_SOURCE
+#define _DEFAULT_SOURCE
+#endif
+
 #ifndef _POSIX_SOURCE
 #define _POSIX_SOURCE
 #endif
@@ -27,7 +31,7 @@
 #include <time.h>
 #include <signal.h>
 #include <pthread.h>
-#include <bits/pthreadtypes.h>
+#include <sys/types.h>
 #include <bits/sigthread.h>
 
 #include "nwipe.h"
@@ -45,8 +49,10 @@
 
 #include <parted/parted.h>
 #include <parted/debug.h>
+#include "version.h"
 
 int terminate_signal;
+int user_abort;
 
 int main( int argc, char** argv )
 {
@@ -67,9 +73,18 @@ int main( int argc, char** argv )
     /* The generic result buffer. */
     int r;
 
+    /* Log nwipes version */
+    nwipe_log( NWIPE_LOG_INFO, "%s", banner );
+
+    /* Log OS info */
+    nwipe_log_OSinfo();
+
     /* Initialise the termintaion signal, 1=terminate nwipe */
     terminate_signal = 0;
 
+    /* Initialise the user abort signal, 1=User aborted with CNTRL-C,SIGTERM, SIGQUIT, SIGINT etc.. */
+    user_abort = 0;
+
     /* nwipes return status value, set prior to exit at the end of nwipe, as no other exit points allowed */
     int return_status = 0;
 
@@ -81,6 +96,8 @@ int main( int argc, char** argv )
     /* Initialised and populated in device scan.     */
     nwipe_context_t** c1 = 0;
 
+    int wipe_threads_started = 0;
+
     /* Parse command line options. */
     nwipe_optind = nwipe_options_parse( argc, argv );
     if( nwipe_optind == argc )
@@ -90,7 +107,8 @@ int main( int argc, char** argv )
 
         if( nwipe_enumerated == 0 )
         {
-            nwipe_log( NWIPE_LOG_INFO, "Storage devices not found." );
+            nwipe_log( NWIPE_LOG_INFO,
+                       "Storage devices not found. Nwipe should be run as root or sudo/su, i.e sudo nwipe etc" );
             cleanup();
             return -1;
         }
@@ -108,12 +126,18 @@ int main( int argc, char** argv )
         if( nwipe_enumerated == 0 )
         {
             nwipe_log( NWIPE_LOG_ERROR, "Devices not found. Check you're not excluding drives unnecessarily." );
-            printf( "No drives found" );
+            printf( "No drives found\n" );
             cleanup();
             exit( 1 );
         }
     }
 
+    if( terminate_signal == 1 )
+    {
+        cleanup();
+        exit( 1 );
+    }
+
     /* Log the System information */
     nwipe_log_sysinfo();
 
@@ -129,6 +153,7 @@ int main( int argc, char** argv )
         nwipe_perror( errno, __FUNCTION__, "open" );
         nwipe_log( NWIPE_LOG_FATAL, "Unable to open entropy source %s.", NWIPE_KNOB_ENTROPY );
         cleanup();
+        free( c2 );
         return errno;
     }
 
@@ -255,150 +280,166 @@ int main( int argc, char** argv )
     }
 
     /* TODO: free c1 and c2 memory. */
-
-    for( i = 0; i < nwipe_selected; i++ )
+    if( user_abort == 0 )
     {
-        /* A result buffer for the BLKGETSIZE64 ioctl. */
-        u64 size64;
-
-        /* Initialise the wipe_status flag, -1 = wipe not yet started */
-        c2[i]->wipe_status = -1;
-
-        /* Open the file for reads and writes. */
-        c2[i]->device_fd = open( c2[i]->device_name, O_RDWR );
-
-        /* Check the open() result. */
-        if( c2[i]->device_fd < 0 )
+        for( i = 0; i < nwipe_selected; i++ )
         {
-            nwipe_perror( errno, __FUNCTION__, "open" );
-            nwipe_log( NWIPE_LOG_WARNING, "Unable to open device '%s'.", c2[i]->device_name );
-            c2[i]->select = NWIPE_SELECT_DISABLED;
-            continue;
-        }
+            /* A result buffer for the BLKGETSIZE64 ioctl. */
+            u64 size64;
 
-        /* Stat the file. */
-        if( fstat( c2[i]->device_fd, &c2[i]->device_stat ) != 0 )
-        {
-            nwipe_perror( errno, __FUNCTION__, "fstat" );
-            nwipe_log( NWIPE_LOG_ERROR, "Unable to stat file '%s'.", c2[i]->device_name );
-            nwipe_error++;
-            continue;
-        }
+            /* Initialise the spinner character index */
+            c2[i]->spinner_idx = 0;
 
-        /* Check that the file is a block device. */
-        if( !S_ISBLK( c2[i]->device_stat.st_mode ) )
-        {
-            nwipe_log( NWIPE_LOG_ERROR, "'%s' is not a block device.", c2[i]->device_name );
-            nwipe_error++;
-            continue;
-        }
+            /* Initialise the start and end time of the wipe */
+            c2[i]->start_time = 0;
+            c2[i]->end_time = 0;
 
-        /* TODO: Lock the file for exclusive access. */
-        /*
-        if( flock( c2[i]->device_fd, LOCK_EX | LOCK_NB ) != 0 )
-        {
-                nwipe_perror( errno, __FUNCTION__, "flock" );
-                nwipe_log( NWIPE_LOG_ERROR, "Unable to lock the '%s' file.", c2[i]->device_name );
+            /* Initialise the wipe_status flag, -1 = wipe not yet started */
+            c2[i]->wipe_status = -1;
+
+            /* Open the file for reads and writes. */
+            c2[i]->device_fd = open( c2[i]->device_name, O_RDWR );
+
+            /* Check the open() result. */
+            if( c2[i]->device_fd < 0 )
+            {
+                nwipe_perror( errno, __FUNCTION__, "open" );
+                nwipe_log( NWIPE_LOG_WARNING, "Unable to open device '%s'.", c2[i]->device_name );
+                c2[i]->select = NWIPE_SELECT_DISABLED;
+                continue;
+            }
+
+            /* Stat the file. */
+            if( fstat( c2[i]->device_fd, &c2[i]->device_stat ) != 0 )
+            {
+                nwipe_perror( errno, __FUNCTION__, "fstat" );
+                nwipe_log( NWIPE_LOG_ERROR, "Unable to stat file '%s'.", c2[i]->device_name );
                 nwipe_error++;
                 continue;
-        }
-        */
+            }
 
-        /* Print serial number of device if it exists. */
-        if( strlen( (const char*) c2[i]->device_serial_no ) )
-        {
-            nwipe_log( NWIPE_LOG_INFO, "Device %s has serial number %s", c2[i]->device_name, c2[i]->device_serial_no );
-        }
-
-        /* Do sector size and block size checking. */
-        if( ioctl( c2[i]->device_fd, BLKSSZGET, &c2[i]->device_sector_size ) == 0 )
-        {
-            nwipe_log(
-                NWIPE_LOG_INFO, "Device '%s' has sector size %i.", c2[i]->device_name, c2[i]->device_sector_size );
-
-            if( ioctl( c2[i]->device_fd, BLKBSZGET, &c2[i]->device_block_size ) == 0 )
+            /* Check that the file is a block device. */
+            if( !S_ISBLK( c2[i]->device_stat.st_mode ) )
             {
-                nwipe_log(
-                    NWIPE_LOG_INFO, "Device '%s' has block size %i.", c2[i]->device_name, c2[i]->device_block_size );
+                nwipe_log( NWIPE_LOG_ERROR, "'%s' is not a block device.", c2[i]->device_name );
+                nwipe_error++;
+                continue;
+            }
+
+            /* TODO: Lock the file for exclusive access. */
+            /*
+            if( flock( c2[i]->device_fd, LOCK_EX | LOCK_NB ) != 0 )
+            {
+                    nwipe_perror( errno, __FUNCTION__, "flock" );
+                    nwipe_log( NWIPE_LOG_ERROR, "Unable to lock the '%s' file.", c2[i]->device_name );
+                    nwipe_error++;
+                    continue;
+            }
+            */
+
+            /* Print serial number of device if it exists. */
+            if( strlen( (const char*) c2[i]->device_serial_no ) )
+            {
+                nwipe_log( NWIPE_LOG_NOTICE, "%s has serial number %s", c2[i]->device_name, c2[i]->device_serial_no );
+            }
+
+            /* Do sector size and block size checking. */
+            if( ioctl( c2[i]->device_fd, BLKSSZGET, &c2[i]->device_sector_size ) == 0 )
+            {
+
+                if( ioctl( c2[i]->device_fd, BLKBSZGET, &c2[i]->device_block_size ) != 0 )
+                {
+                    nwipe_log( NWIPE_LOG_WARNING, "Device '%s' failed BLKBSZGET ioctl.", c2[i]->device_name );
+                    c2[i]->device_block_size = 0;
+                }
             }
             else
             {
-                nwipe_log( NWIPE_LOG_WARNING, "Device '%s' failed BLKBSZGET ioctl.", c2[i]->device_name );
+                nwipe_log( NWIPE_LOG_WARNING, "Device '%s' failed BLKSSZGET ioctl.", c2[i]->device_name );
+                c2[i]->device_sector_size = 0;
                 c2[i]->device_block_size = 0;
             }
-        }
-        else
-        {
-            nwipe_log( NWIPE_LOG_WARNING, "Device '%s' failed BLKSSZGET ioctl.", c2[i]->device_name );
-            c2[i]->device_sector_size = 0;
-            c2[i]->device_block_size = 0;
-        }
 
-        /* The st_size field is zero for block devices. */
-        /* ioctl( c2[i]->device_fd, BLKGETSIZE64, &c2[i]->device_size ); */
+            /* The st_size field is zero for block devices. */
+            /* ioctl( c2[i]->device_fd, BLKGETSIZE64, &c2[i]->device_size ); */
 
-        /* Seek to the end of the device to determine its size. */
-        c2[i]->device_size = lseek( c2[i]->device_fd, 0, SEEK_END );
+            /* Seek to the end of the device to determine its size. */
+            c2[i]->device_size = lseek( c2[i]->device_fd, 0, SEEK_END );
 
-        /* Also ask the driver for the device size. */
-        /* if( ioctl( c2[i]->device_fd, BLKGETSIZE64, &size64 ) ) */
-        if( ioctl( c2[i]->device_fd, _IOR( 0x12, 114, size_t ), &size64 ) )
-        {
-            /* The ioctl failed. */
-            fprintf( stderr, "Error: BLKGETSIZE64 failed  on '%s'.\n", c2[i]->device_name );
-            nwipe_log( NWIPE_LOG_ERROR, "BLKGETSIZE64 failed  on '%s'.\n", c2[i]->device_name );
-            nwipe_error++;
-        }
-        c2[i]->device_size = size64;
-
-        /* Check whether the two size values agree. */
-        if( c2[i]->device_size != size64 )
-        {
-            /* This could be caused by the linux last-odd-block problem. */
-            fprintf( stderr, "Error: Last-odd-block detected on '%s'.\n", c2[i]->device_name );
-            nwipe_log( NWIPE_LOG_ERROR, "Last-odd-block detected on '%s'.", c2[i]->device_name );
-            nwipe_error++;
-        }
-
-        if( c2[i]->device_size == (long long) -1 )
-        {
-            /* We cannot determine the size of this device. */
-            nwipe_perror( errno, __FUNCTION__, "lseek" );
-            nwipe_log( NWIPE_LOG_ERROR, "Unable to determine the size of '%s'.", c2[i]->device_name );
-            nwipe_error++;
-        }
-        else
-        {
-            /* Reset the file pointer. */
-            r = lseek( c2[i]->device_fd, 0, SEEK_SET );
-
-            if( r == (off64_t) -1 )
+            /* Also ask the driver for the device size. */
+            /* if( ioctl( c2[i]->device_fd, BLKGETSIZE64, &size64 ) ) */
+            if( ioctl( c2[i]->device_fd, _IOR( 0x12, 114, size_t ), &size64 ) )
             {
-                nwipe_perror( errno, __FUNCTION__, "lseek" );
-                nwipe_log( NWIPE_LOG_ERROR, "Unable to reset the '%s' file offset.", c2[i]->device_name );
+                /* The ioctl failed. */
+                fprintf( stderr, "Error: BLKGETSIZE64 failed  on '%s'.\n", c2[i]->device_name );
+                nwipe_log( NWIPE_LOG_ERROR, "BLKGETSIZE64 failed  on '%s'.\n", c2[i]->device_name );
                 nwipe_error++;
             }
-        }
+            c2[i]->device_size = size64;
 
-        if( c2[i]->device_size == 0 )
-        {
-            nwipe_log( NWIPE_LOG_ERROR, "Device '%s' is size %llu.", c2[i]->device_name, c2[i]->device_size );
-            nwipe_error++;
-            continue;
-        }
-        else
-        {
-            nwipe_log( NWIPE_LOG_INFO, "Device '%s' is size %llu.", c2[i]->device_name, c2[i]->device_size );
-        }
+            /* Check whether the two size values agree. */
+            if( c2[i]->device_size != size64 )
+            {
+                /* This could be caused by the linux last-odd-block problem. */
+                fprintf( stderr, "Error: Last-odd-block detected on '%s'.\n", c2[i]->device_name );
+                nwipe_log( NWIPE_LOG_ERROR, "Last-odd-block detected on '%s'.", c2[i]->device_name );
+                nwipe_error++;
+            }
 
-        /* Fork a child process. */
-        errno = pthread_create( &c2[i]->thread, NULL, nwipe_options.method, (void*) c2[i] );
-        if( errno )
-        {
-            nwipe_perror( errno, __FUNCTION__, "pthread_create" );
-            if( !nwipe_options.nogui )
-                nwipe_gui_free();
-            return errno;
+            if( c2[i]->device_size == (long long) -1 )
+            {
+                /* We cannot determine the size of this device. */
+                nwipe_perror( errno, __FUNCTION__, "lseek" );
+                nwipe_log( NWIPE_LOG_ERROR, "Unable to determine the size of '%s'.", c2[i]->device_name );
+                nwipe_error++;
+            }
+            else
+            {
+                /* Reset the file pointer. */
+                r = lseek( c2[i]->device_fd, 0, SEEK_SET );
+
+                if( r == (off64_t) -1 )
+                {
+                    nwipe_perror( errno, __FUNCTION__, "lseek" );
+                    nwipe_log( NWIPE_LOG_ERROR, "Unable to reset the '%s' file offset.", c2[i]->device_name );
+                    nwipe_error++;
+                }
+            }
+
+            if( c2[i]->device_size == 0 )
+            {
+                nwipe_log( NWIPE_LOG_ERROR,
+                           "%s, sect/blk/dev %llu/%i/%llu",
+                           c2[i]->device_name,
+                           c2[i]->device_sector_size,
+                           c2[i]->device_block_size,
+                           c2[i]->device_size );
+                nwipe_error++;
+                continue;
+            }
+            else
+            {
+                nwipe_log( NWIPE_LOG_NOTICE,
+                           "%s, sect/blk/dev %llu/%i/%llu",
+                           c2[i]->device_name,
+                           c2[i]->device_sector_size,
+                           c2[i]->device_block_size,
+                           c2[i]->device_size );
+            }
+
+            /* Fork a child process. */
+            errno = pthread_create( &c2[i]->thread, NULL, nwipe_options.method, (void*) c2[i] );
+            if( errno )
+            {
+                nwipe_perror( errno, __FUNCTION__, "pthread_create" );
+                if( !nwipe_options.nogui )
+                    nwipe_gui_free();
+                return errno;
+            }
+            else
+            {
+                wipe_threads_started = 1;
+            }
         }
     }
 
@@ -419,6 +460,10 @@ int main( int argc, char** argv )
     }
 
     /* Wait for all the wiping threads to finish, but don't wait if we receive the terminate signal */
+
+    /* set getch delay to 2/10th second. */
+    halfdelay( 10 );
+
     i = 0;
     while( i < nwipe_selected && terminate_signal == 0 )
     {
@@ -436,33 +481,35 @@ int main( int argc, char** argv )
             i++;
             continue;
         }
-        sleep( 2 ); /* DO NOT REMOVE ! Stops the routine hogging CPU cycles */
+        sleep( 1 ); /* DO NOT REMOVE ! Stops the routine hogging CPU cycles */
     }
 
-    if( terminate_signal == 1 )
+    if( terminate_signal != 1 )
     {
-        nwipe_log( NWIPE_LOG_INFO, "Program interrupted" );
-    }
-    else
-    {
-        if( !nwipe_options.nowait )
+        if( !nwipe_options.nowait && !nwipe_options.autopoweroff )
         {
             do
             {
                 sleep( 1 );
+
             } while( terminate_signal != 1 );
         }
     }
-    nwipe_log( NWIPE_LOG_INFO, "Exit in progress" );
-
+    if( nwipe_options.verbose )
+    {
+        nwipe_log( NWIPE_LOG_INFO, "Exit in progress" );
+    }
     /* Send a REQUEST for the wipe threads to be cancelled */
     for( i = 0; i < nwipe_selected; i++ )
     {
 
         if( c2[i]->thread )
         {
-            nwipe_log( NWIPE_LOG_INFO, "Requesting wipe thread cancellation for %s", c2[i]->device_name );
-            nwipe_log( NWIPE_LOG_INFO, "Please wait.." );
+            if( nwipe_options.verbose )
+            {
+                nwipe_log( NWIPE_LOG_INFO, "Requesting wipe thread cancellation for %s", c2[i]->device_name );
+                nwipe_log( NWIPE_LOG_INFO, "Please wait.." );
+            }
             pthread_cancel( c2[i]->thread );
         }
     }
@@ -470,7 +517,10 @@ int main( int argc, char** argv )
     /* Kill the GUI thread */
     if( nwipe_gui_thread )
     {
-        nwipe_log( NWIPE_LOG_INFO, "Cancelling the GUI thread." );
+        if( nwipe_options.verbose )
+        {
+            nwipe_log( NWIPE_LOG_INFO, "Cancelling the GUI thread." );
+        }
 
         /* We don't want to use pthread_cancel as our GUI thread is aware of the control-c
          *  signal and will exit itself we just join the GUI thread and wait for confirmation
@@ -480,7 +530,10 @@ int main( int argc, char** argv )
         {
             nwipe_log( NWIPE_LOG_WARNING, "main()>pthread_join():Error when waiting for GUI thread to cancel." );
         }
-        nwipe_log( NWIPE_LOG_INFO, "GUI compute_stats thread has been cancelled" );
+        if( nwipe_options.verbose )
+        {
+            nwipe_log( NWIPE_LOG_INFO, "GUI compute_stats thread has been cancelled" );
+        }
     }
 
     /* Release the gui. */
@@ -502,31 +555,51 @@ int main( int argc, char** argv )
                 nwipe_log( NWIPE_LOG_WARNING, "main()>pthread_join():Error when waiting for wipe thread to cancel." );
             }
             c2[i]->thread = 0; /* Zero the thread so we know it's been cancelled */
-            nwipe_log( NWIPE_LOG_INFO, "Wipe thread for device %s has been cancelled", c2[i]->device_name );
+
+            if( nwipe_options.verbose )
+            {
+                nwipe_log( NWIPE_LOG_INFO, "Wipe thread for device %s has been cancelled", c2[i]->device_name );
+            }
+
             /* Close the device file descriptor. */
             close( c2[i]->device_fd );
         }
     }
 
-    for( i = 0; i < nwipe_selected; i++ )
+    /* if no wipe threads started then zero each selected drive result flag,
+     * as we don't need to report fatal/non fatal errors if no wipes were ever started ! */
+    if( wipe_threads_started == 0 )
     {
-        /* Check for non-fatal errors. */
-        if( c2[i]->result > 0 )
+        for( i = 0; i < nwipe_selected; i++ )
         {
-            nwipe_log( NWIPE_LOG_FATAL, "Nwipe exited with non fatal errors on device = %s\n", c2[i]->device_name );
-            return_status = 1;
+            c2[i]->result = 0;
+        }
+    }
+    else
+    {
+        for( i = 0; i < nwipe_selected; i++ )
+        {
+            /* Check for non-fatal errors. */
+            if( c2[i]->result > 0 )
+            {
+                nwipe_log( NWIPE_LOG_FATAL, "Nwipe exited with non fatal errors on device = %s\n", c2[i]->device_name );
+                return_status = 1;
+            }
+        }
+
+        for( i = 0; i < nwipe_selected; i++ )
+        {
+            /* Check for fatal errors. */
+            if( c2[i]->result < 0 )
+            {
+                nwipe_log( NWIPE_LOG_ERROR, "Nwipe exited with fatal errors on device = %s\n", c2[i]->device_name );
+                return_status = -1;
+            }
         }
     }
 
-    for( i = 0; i < nwipe_selected; i++ )
-    {
-        /* Check for fatal errors. */
-        if( c2[i]->result < 0 )
-        {
-            nwipe_log( NWIPE_LOG_ERROR, "Nwipe exited with fatal errors on device = %s\n", c2[i]->device_name );
-            return_status = -1;
-        }
-    }
+    /* Generate and send the drive status summary to the log */
+    nwipe_log_summary( c2, nwipe_selected );
 
     if( return_status == 0 )
     {
@@ -535,6 +608,8 @@ int main( int argc, char** argv )
 
     cleanup();
 
+    check_for_autopoweroff();
+
     /* Exit. */
     return return_status;
 }
@@ -542,6 +617,13 @@ int main( int argc, char** argv )
 void* signal_hand( void* ptr )
 {
     int sig;
+    int hours;
+    int minutes;
+    int seconds;
+
+    hours = 0;
+    minutes = 0;
+    seconds = 0;
 
     // Define signals that this handler should react to
     sigset_t sigset;
@@ -553,6 +635,7 @@ void* signal_hand( void* ptr )
     sigaddset( &sigset, SIGUSR1 );
 
     int i;
+    char eta[9];
 
     /* Set up the structs we will use for the data required. */
     nwipe_thread_data_ptr_t* nwipe_thread_data_ptr;
@@ -574,7 +657,6 @@ void* signal_hand( void* ptr )
 
             // Log current status. All values are automatically updated by the GUI
             case SIGUSR1:
-            {
                 compute_stats( ptr );
 
                 for( i = 0; i < nwipe_misc_thread_data->nwipe_selected; i++ )
@@ -608,14 +690,20 @@ void* signal_hand( void* ptr )
                         {
                             status = "[syncing]";
                         }
+
+                        convert_seconds_to_hours_minutes_seconds( c[i]->eta, &hours, &minutes, &seconds );
+
                         nwipe_log( NWIPE_LOG_INFO,
-                                   "%s: %05.2f%%, round %i of %i, pass %i of %i %s",
+                                   "%s: %05.2f%%, round %i of %i, pass %i of %i, eta %02i:%02i:%02i, %s",
                                    c[i]->device_name,
                                    c[i]->round_percent,
                                    c[i]->round_working,
                                    c[i]->round_count,
                                    c[i]->pass_working,
                                    c[i]->pass_count,
+                                   hours,
+                                   minutes,
+                                   seconds,
                                    status );
                     }
                     else
@@ -638,21 +726,21 @@ void* signal_hand( void* ptr )
                 }
 
                 break;
-            }
 
             case SIGHUP:
             case SIGINT:
             case SIGQUIT:
             case SIGTERM:
-            {
                 /* Set termination flag for main() which will do housekeeping prior to exit */
                 terminate_signal = 1;
 
+                /* Set the user abort flag */
+                user_abort = 1;
+
                 /* Return control to the main thread, returning the signal received */
                 return ( (void*) 0 );
 
                 break;
-            }
         }
     }
 
@@ -688,3 +776,21 @@ int cleanup()
 
     return 0;
 }
+void check_for_autopoweroff( void )
+{
+    char cmd[] = "shutdown -P +1 \"System going down in one minute\"";
+    FILE* fp;
+    int r;  // A result buffer.
+
+    /* User request auto power down ? */
+    if( nwipe_options.autopoweroff == 1 )
+    {
+        fp = popen( cmd, "r" );
+        if( fp == NULL )
+        {
+            nwipe_log( NWIPE_LOG_INFO, "Failed to autopoweroff to %s", cmd );
+            return;
+        }
+        r = pclose( fp );
+    }
+}
