@@ -53,13 +53,16 @@ int nwipe_options_parse( int argc, char** argv )
     int i;
 
     /* The list of acceptable short options. */
-    char nwipe_options_short[] = "Vhl:m:p:r:e:";
+    char nwipe_options_short[] = "Vvhl:m:p:r:e:";
 
     /* The list of acceptable long options. */
     static struct option nwipe_options_long[] = {
         /* Set when the user wants to wipe without a confirmation prompt. */
         {"autonuke", no_argument, 0, 0},
 
+        /* Set when the user wants to have the system powerdown on completion of wipe. */
+        {"autopoweroff", no_argument, 0, 0},
+
         /* A GNU standard option. Corresponds to the 'h' short option. */
         {"help", no_argument, 0, 'h'},
 
@@ -81,6 +84,9 @@ int nwipe_options_parse( int argc, char** argv )
         /* Whether to blank the disk after wiping. */
         {"noblank", no_argument, 0, 0},
 
+        /* Whether to ignore all USB devices. */
+        {"nousb", no_argument, 0, 0},
+
         /* Whether to exit after wiping or wait for a keypress. */
         {"nowait", no_argument, 0, 0},
 
@@ -96,6 +102,9 @@ int nwipe_options_parse( int argc, char** argv )
         /* Verify that wipe patterns are being written to the device. */
         {"verify", required_argument, 0, 0},
 
+        /* Display program version. */
+        {"verbose", no_argument, 0, 'v'},
+
         /* Display program version. */
         {"version", no_argument, 0, 'V'},
 
@@ -104,14 +113,17 @@ int nwipe_options_parse( int argc, char** argv )
 
     /* Set default options. */
     nwipe_options.autonuke = 0;
+    nwipe_options.autopoweroff = 0;
     nwipe_options.method = &nwipe_dodshort;
     nwipe_options.prng = &nwipe_twister;
     nwipe_options.rounds = 1;
     nwipe_options.noblank = 0;
+    nwipe_options.nousb = 0;
     nwipe_options.nowait = 0;
     nwipe_options.nosignals = 0;
     nwipe_options.nogui = 0;
-    nwipe_options.sync = 100000;
+    nwipe_options.sync = DEFAULT_SYNC_RATE;
+    nwipe_options.verbose = 0;
     nwipe_options.verify = NWIPE_VERIFY_LAST;
     memset( nwipe_options.logfile, '\0', sizeof( nwipe_options.logfile ) );
 
@@ -143,12 +155,24 @@ int nwipe_options_parse( int argc, char** argv )
                     break;
                 }
 
+                if( strcmp( nwipe_options_long[i].name, "autopoweroff" ) == 0 )
+                {
+                    nwipe_options.autopoweroff = 1;
+                    break;
+                }
+
                 if( strcmp( nwipe_options_long[i].name, "noblank" ) == 0 )
                 {
                     nwipe_options.noblank = 1;
                     break;
                 }
 
+                if( strcmp( nwipe_options_long[i].name, "nousb" ) == 0 )
+                {
+                    nwipe_options.nousb = 1;
+                    break;
+                }
+
                 if( strcmp( nwipe_options_long[i].name, "nowait" ) == 0 )
                 {
                     nwipe_options.nowait = 1;
@@ -168,11 +192,17 @@ int nwipe_options_parse( int argc, char** argv )
                     break;
                 }
 
+                if( strcmp( nwipe_options_long[i].name, "verbose" ) == 0 )
+                {
+                    nwipe_options.verbose = 1;
+                    break;
+                }
+
                 if( strcmp( nwipe_options_long[i].name, "sync" ) == 0 )
                 {
-                    if( sscanf( optarg, " %i", &nwipe_options.sync ) != 1 || nwipe_options.sync < 1 )
+                    if( sscanf( optarg, " %i", &nwipe_options.sync ) != 1 || nwipe_options.sync < 0 )
                     {
-                        fprintf( stderr, "Error: The sync argument must be a positive integer.\n" );
+                        fprintf( stderr, "Error: The sync argument must be a positive integer or zero.\n" );
                         exit( EINVAL );
                     }
                     break;
@@ -343,6 +373,11 @@ int nwipe_options_parse( int argc, char** argv )
 
                 break;
 
+            case 'v': /* verbose */
+
+                nwipe_options.verbose = 1;
+                break;
+
             case 'V': /* Version option. */
 
                 printf( "%s version %s\n", program_name, version_string );
@@ -379,6 +414,15 @@ void nwipe_options_log( void )
         nwipe_log( NWIPE_LOG_NOTICE, "  autonuke = %i (off)", nwipe_options.autonuke );
     }
 
+    if( nwipe_options.autopoweroff )
+    {
+        nwipe_log( NWIPE_LOG_NOTICE, "  autopoweroff = %i (on)", nwipe_options.autopoweroff );
+    }
+    else
+    {
+        nwipe_log( NWIPE_LOG_NOTICE, "  autopoweroff = %i (off)", nwipe_options.autopoweroff );
+    }
+
     if( nwipe_options.noblank )
     {
         nwipe_log( NWIPE_LOG_NOTICE, "  do not perform a final blank pass" );
@@ -435,15 +479,23 @@ void display_help()
     /* Limit line length to a maximum of 80 characters so it looks good in 80x25 terminals i.e shredos */
     /*  ___12345678901234567890123456789012345678901234567890123456789012345678901234567890< Do not exceed */
     puts( "  -V, --version           Prints the version number\n" );
+    puts( "  -v, --verbose           Prints more messages to the log\n" );
     puts( "  -h, --help              Prints this help\n" );
     puts( "      --autonuke          If no devices have been specified on the command line," );
     puts( "                          starts wiping all devices immediately. If devices have" );
     puts( "                          been specified, starts wiping only those specified" );
     puts( "                          devices immediately.\n" );
-    puts( "      --sync=NUM          Will perform a sync after NUM writes (default: 0)" );
+    puts( "      --autopoweroff      Power off system on completion of wipe delayed for" );
+    puts( "                          for one minute. During this one minute delay you can" );
+    puts( "                          abort the shutdown by typing sudo shutdown -c\n" );
+    printf( "      --sync=NUM          Will perform a sync after NUM writes (default: %d)\n", DEFAULT_SYNC_RATE );
     puts( "                          0 - fdatasync after the disk is completely written" );
+    puts( "                              fdatasync errors not detected until completion." );
+    puts( "                              0 is not recommended as disk errors may cause nwipe" );
+    puts( "                              to appear to hang" );
     puts( "                          1 - fdatasync after every write" );
-    puts( "                          1000000 - fdatasync after 1000000 writes ect." );
+    puts( "                              Warning: Lower values will reduce wipe speeds." );
+    puts( "                          1000000 - fdatasync after 1000000 writes etc.)\n" );
     puts( "      --verify=TYPE       Whether to perform verification of erasure" );
     puts( "                          (default: last)" );
     puts( "                          off   - Do not verify" );
@@ -471,6 +523,8 @@ void display_help()
     puts( "      --nogui             Do not show the GUI interface. Automatically invokes" );
     puts( "                          the nowait option. Must be used with the --autonuke" );
     puts( "                          option. Send SIGUSR1 to log current stats\n" );
+    puts( "      --nousb             Do show or wipe any USB devices whether in GUI" );
+    puts( "                          mode, --nogui or --autonuke modes.\n" );
     puts( "  -e, --exclude=DEVICES   Up to ten comma separated devices to be excluded" );
     puts( "                          --exclude=/dev/sdc" );
     puts( "                          --exclude=/dev/sdc,/dev/sdd" );
